library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;

ENTITY BlinkingLed_Fpga IS
PORT (
  clk: IN BIT;
  reset_Pin : IN BIT;
  led1 : OUT BIT;
  led2 : OUT BIT;
  led3 : OUT BIT;
  led4 : OUT BIT;
  led5 : OUT BIT;
  led6 : OUT BIT;
  led7 : OUT BIT;
  led8 : OUT BIT  --last line of port definition
);
END BlinkingLed_Fpga;

ARCHITECTURE BEHAVIORAL OF BlinkingLed_Fpga IS

-- The external VHDL file RAM_SpiRamSel is defined here. 
--  Its file and entity name comes from annotation "@Fpga.VHDL_MODULE ( vhdlEntity = <name>" in the Java implementation file.
--  The PORT variables comes from the implementation Java file, which should match to the VHDL file.
COMPONENT RAM_SpiRamSel
PORT (
  Clock: IN STD_LOGIC ;
  ClockEn: IN STD_LOGIC ;
  Reset: IN STD_LOGIC ;
  WE: IN STD_LOGIC ;
  Address: IN STD_LOGIC_VECTOR(6 DOWNTO 0) ;
  Data: IN STD_LOGIC_VECTOR(7 DOWNTO 0) ;
  Q: OUT STD_LOGIC_VECTOR(7 DOWNTO 0) 
 );
END COMPONENT;

-- The external VHDL file Test_Combinatoric is defined here. 
--  Its file and entity name comes from annotation "@Fpga.VHDL_MODULE ( vhdlEntity = <name>" in the Java implementation file.
--  The PORT variables comes from the implementation Java file, which should match to the VHDL file.
COMPONENT Test_Combinatoric
PORT (
  S: IN BIT ;
  R: IN BIT ;
  Q1: OUT BIT ;
  Q2: OUT BIT 
 );
END COMPONENT;

TYPE BlinkingLedCt_Q_REC IS RECORD
  ctLow : STD_LOGIC_VECTOR(15 DOWNTO 0);
  ct : STD_LOGIC_VECTOR(7 DOWNTO 0);
  led : BIT;
  state : BIT_VECTOR(3 DOWNTO 0);
END RECORD BlinkingLedCt_Q_REC;

TYPE BlinkingLedCt_Vhdlink_ram_REC IS RECORD
  we : BIT;
  test1 : BIT;
  test2 : BIT;
  test3 : BIT;
  test4 : BIT;
  ramAddr : STD_LOGIC_VECTOR(6 DOWNTO 0);
  ramData : STD_LOGIC_VECTOR(7 DOWNTO 0);
END RECORD BlinkingLedCt_Vhdlink_ram_REC;

TYPE BlinkingLedCt_RamUse_REC IS RECORD
  outRam : BIT;
  sh : BIT_VECTOR(7 DOWNTO 0);
END RECORD BlinkingLedCt_RamUse_REC;

TYPE BlinkingLed_Fpga_Vhdlink_vhdl_Combinatoric_REC IS RECORD
  S : BIT;
  R : BIT;
  Q1 : BIT;
  Q2 : BIT;
END RECORD BlinkingLed_Fpga_Vhdlink_vhdl_Combinatoric_REC;

TYPE BlinkingLed_Fpga_TestOut_REC IS RECORD
  testOut1 : BIT;
  testOut2 : BIT;
  testOut3 : BIT;
END RECORD BlinkingLed_Fpga_TestOut_REC;

TYPE ClockDivider_Q_REC IS RECORD
  ct : STD_LOGIC_VECTOR(3 DOWNTO 0);
  ce : BIT;
END RECORD ClockDivider_Q_REC;

TYPE ResetCe_Q_REC IS RECORD
  res : BIT;
  res1 : BIT;
END RECORD ResetCe_Q_REC;

-- == Definition of record instances for the processes:
-- VHDL-Syntax:          SIGNAL <signalName> : <typeName>
--SIGNAL {<&nameModule>_}<&nameProcessClass> : <&nameClass>_<&nameProcessClass>_REC
--   ... <&nameModule> is defined in  class Modules{...}  in top level Java file, name of the module instance.
--   ... <&nameModule> for Process classes in toplevel: name of the top level class and Java file (start with upper case).
--   ... more <&nameModule> can be defined in  class Modules{...}  in the module Java file, nested modules.
--   ... <&nameProcessClass> is the name of the inner class in the Java module, same as instance name of the process
--   ... <&nameClass> is the type name of the modul's class
SIGNAL BlinkingLed_Fpga_Vhdlink_vhdl_Combinatoric : BlinkingLed_Fpga_Vhdlink_vhdl_Combinatoric_REC;
SIGNAL BlinkingLed_Fpga_TestOut : BlinkingLed_Fpga_TestOut_REC;
SIGNAL ce_Q : ClockDivider_Q_REC;
SIGNAL ct_Q : BlinkingLedCt_Q_REC;
SIGNAL ct_Vhdlink_ram : BlinkingLedCt_Vhdlink_ram_REC;
SIGNAL ct_RamUse : BlinkingLedCt_RamUse_REC;
SIGNAL res_Q : ResetCe_Q_REC;

CONSTANT BlinkingLed_Fpga_blinkingLedCfg_onDuration_BlinkingLed : INTEGER := 10;
CONSTANT BlinkingLed_Fpga_blinkingLedCfg_time : INTEGER := 0;
CONSTANT BlinkingLed_Fpga_blinkingLedCfg_time_BlinkingLed : BIT_VECTOR(7 DOWNTO 0) := x"64";
CONSTANT ClockDivider_ce_period : INTEGER := 10;
CONSTANT ResetCe_stubFalse_getBit : BIT := '0';
CONSTANT ResetCe_stubTrue_getBit : BIT := '1';
CONSTANT ResetCe_stubWord16_getWord : BIT_VECTOR(15 DOWNTO 0) := x"0000";
CONSTANT BlinkingLedCt_BlinkingLedCt_State_fast : BIT_VECTOR(3 DOWNTO 0) := x"1";
CONSTANT BlinkingLedCt_BlinkingLedCt_State_nonInit : BIT_VECTOR(3 DOWNTO 0) := x"0";
CONSTANT BlinkingLedCt_BlinkingLedCt_State_off : BIT_VECTOR(3 DOWNTO 0) := x"4";
CONSTANT BlinkingLedCt_BlinkingLedCt_State_slow : BIT_VECTOR(3 DOWNTO 0) := x"2";



BEGIN

-- The external VHDL file Test_Combinatoric is included here. 

-- Assignments for VHDL instance inputs:
BlinkingLed_Fpga_Vhdlink_vhdl_Combinatoric.S <= TO_BIT(ct_Q.ctLow(13) AND ct_Q.ctLow(11) );
BlinkingLed_Fpga_Vhdlink_vhdl_Combinatoric.R <= TO_BIT(ct_Q.ctLow(13) AND ct_Q.ctLow(10) );

-- <name>: <type> 
--   ... <name> is build from <moduleName>_<className> with annotation "@Fpga.LINK_VHDL_MODULE" 
--   ... <type> is gotten from the Java class implementation annotation "Fpga.VHDL_MODULE ( vhdlEntity = <type> )"
BlinkingLed_Fpga_Vhdlink_vhdl_Combinatoric_vhdlMdl: Test_Combinatoric
PORT MAP( 
  S =>   BlinkingLed_Fpga_Vhdlink_vhdl_Combinatoric.S ,
  R =>   BlinkingLed_Fpga_Vhdlink_vhdl_Combinatoric.R ,
  Q1 => BlinkingLed_Fpga_Vhdlink_vhdl_Combinatoric.Q1  ,
  Q2 => BlinkingLed_Fpga_Vhdlink_vhdl_Combinatoric.Q2  
 );  --PORT MAP Test_Combinatoric

-- The external VHDL file RAM_SpiRamSel is included here. 

-- Assignments for VHDL instance inputs:
ct_Vhdlink_ram.test1 <= TO_BIT(ct_Q.ctLow(15));
ct_Vhdlink_ram.test2 <= TO_BIT(ct_Q.ctLow(14));
ct_Vhdlink_ram.test3 <=  ct_Vhdlink_ram.test1 AND ct_Vhdlink_ram.test2 ;
ct_Vhdlink_ram.test4 <= TO_BIT(ct_Q.ctLow(14));
ct_Vhdlink_ram.we <= TO_BIT(ce_Q.ct(0));
ct_Vhdlink_ram.ramAddr  <= ct_Q.ctLow(6 DOWNTO 0)  WHEN ct_Vhdlink_ram.we='1' ELSE ct_Q.ctLow(8 DOWNTO 2) ;

-- <name>: <type> 
--   ... <name> is build from <moduleName>_<className> with annotation "@Fpga.LINK_VHDL_MODULE" 
--   ... <type> is gotten from the Java class implementation annotation "Fpga.VHDL_MODULE ( vhdlEntity = <type> )"
ct_Vhdlink_ram_vhdlMdl: RAM_SpiRamSel
PORT MAP( 
  Address =>   ct_Vhdlink_ram.ramAddr ,
  Clock =>  TO_STDULOGIC(clk) ,
  ClockEn =>  TO_STDULOGIC(ce_Q.ce) ,
  Data =>   ct_Q.ctLow(8 DOWNTO 1) ,
  Reset =>  TO_STDULOGIC(res_Q.res) ,
  WE =>  TO_STDULOGIC(ct_Vhdlink_ram.we) ,
  Q => ct_Vhdlink_ram.ramData  
 );  --PORT MAP RAM_SpiRamSel

--Assignments from all input() operations to set ..In data of modules



BlinkingLed_Fpga_TestOut_PRC: PROCESS ( clk )
  VARIABLE sel : BIT_VECTOR(13 DOWNTO 0);
  VARIABLE chk : BIT_VECTOR(15 DOWNTO 0);
BEGIN IF(clk'event AND clK='1') THEN

  sel :=   "00000000000000";
  chk :=  x"4100";
  sel(7) :=  '0';
  sel(8) := TO_BIT(ct_Q.ct(0));
  IF  ( ( (sel AND chk(13 DOWNTO 0) ) )  /=  "00000000000000" )  AND chk(15 DOWNTO 14) =  "01"   THEN BlinkingLed_Fpga_TestOut.testOut1  <=  '1'; ELSE BlinkingLed_Fpga_TestOut.testOut1  <=  '0'; END IF;
  BlinkingLed_Fpga_TestOut.testOut2 <=  '0';
  BlinkingLed_Fpga_TestOut.testOut3 <=  '0';
END IF; END PROCESS;



ce_Q_PRC: PROCESS ( clk )
BEGIN IF(clk'event AND clK='1') THEN

  IF reset_Pin = '0'  THEN
      ce_Q.ct <=   "0000";
      ce_Q.ce <=  '0';
  ELSE
    IF ce_Q.ct <  "1001"  THEN
        ce_Q.ct <=  ce_Q.ct + 1 ;
        ce_Q.ce <=  '0';
    ELSE
        ce_Q.ct <=   "0000";
        ce_Q.ce <=  '1';
    END IF;
  END IF;
END IF; END PROCESS;



ct_Q_PRC: PROCESS ( clk )
BEGIN IF(clk'event AND clK='1') THEN

  IF (ce_Q.ce)='1' THEN
      IF (res_Q.res)='1' THEN
          ct_Q.ct <= TO_STDLOGICVECTOR(BlinkingLed_Fpga_blinkingLedCfg_time_BlinkingLed);
          ct_Q.ctLow <=  x"0000";
      ELSE
        IF ct_Q.ctLow(15 DOWNTO 13) =  "111"  THEN
            ct_Q.ctLow <=  x"61a7";
            IF ct_Q.ct = x"00"  THEN
                ct_Q.ct <= TO_STDLOGICVECTOR(BlinkingLed_Fpga_blinkingLedCfg_time_BlinkingLed);
            ELSE
                ct_Q.ct <=  ct_Q.ct - 1 ;
            END IF;
            ct_Q.state <=  BlinkingLedCt_BlinkingLedCt_State_fast;
        ELSE
            ct_Q.ctLow <=  ct_Q.ctLow - 1 ;
        END IF;
      END IF;
      IF ct_Q.ct < BlinkingLed_Fpga_blinkingLedCfg_onDuration_BlinkingLed  THEN ct_Q.led  <=  '1'; ELSE ct_Q.led  <=  '0'; END IF;
  ELSE
  END IF;
END IF; END PROCESS;



ct_RamUse_PRC: PROCESS ( clk )
BEGIN IF(clk'event AND clK='1') THEN

  IF (ce_Q.ce)='1' THEN
      IF ct_RamUse.sh = x"00"  THEN
          ct_RamUse.sh <= TO_BITVECTOR(ct_Vhdlink_ram.ramData);
      ELSE
          ct_RamUse.sh <=  ct_RamUse.sh(6 DOWNTO 0) & ('0');
      END IF;
      ct_RamUse.outRam <=  ct_RamUse.sh(7);
  ELSE
  END IF;
END IF; END PROCESS;



res_Q_PRC: PROCESS ( clk )
BEGIN IF(clk'event AND clK='1') THEN

  IF (ce_Q.ce)='1' THEN
      IF reset_Pin = '0'  THEN
          res_Q.res1 <=  '1';
          res_Q.res <=  '1';
      ELSE
        IF res_Q.res1='1' THEN
            res_Q.res1 <=  '0';
        ELSE
            res_Q.res <=  res_Q.res1;
        END IF;
      END IF;
  ELSE
  END IF;
END IF; END PROCESS;

--Assignments from all output() operations to set Out data of modules and set FPGA out pins

led1 <=  ct_Q.led;
led2  <=  '1' WHEN  (ct_Q.ct(2 DOWNTO 0) =  "000" )  ELSE '0';
led3  <=  '1' WHEN ct_Q.ct(2 DOWNTO 0) /=  "000"  ELSE '0';
led4 <=   (ct_Q.state(1)) ;
led5 <=  ct_Vhdlink_ram.test3;
led6 <=  BlinkingLed_Fpga_Vhdlink_vhdl_Combinatoric.Q1;
led7 <=  BlinkingLed_Fpga_Vhdlink_vhdl_Combinatoric.Q2;
led8 <=  BlinkingLed_Fpga_TestOut.testOut1;

END BEHAVIORAL;
