//tag::theClassDef[]
package org.vishia.fpga.exmplBlinkingLed.fpgatop;

import org.vishia.fpga.stdmodules.CeTime_ifc;
import org.vishia.fpga.stdmodules.Reset;
import org.vishia.fpga.stdmodules.ResetCe;
import org.vishia.fpga.Fpga;                        //tagOffStart::
import org.vishia.fpga.FpgaModule_ifc;
//Note: Do not use a package.path.*, not yet supported by Java2Vhdl
import org.vishia.fpga.exmplBlinkingLed.modules.BlinkingLedCfg_ifc;
import org.vishia.fpga.exmplBlinkingLed.modules.BlinkingLedCt;
import org.vishia.fpga.exmplBlinkingLed.modules.ClockDivider;
import org.vishia.fpga.exmplBlinkingLed.modules.Test_Combinatoric_BlinkingLed; //tagOffEnd::

public class BlinkingLed_Fpga implements FpgaModule_ifc {
  
//end::theClassDef[]


//tag::Modules[] 
//tag::ioPins[]
  /**The modules which are part of this Fpga for test. */
  public class Modules {
    
    /**The i/o pins of the top level FPGA should have exact this name ioPins. */
    public BlinkingLed_FpgaInOutput ioPins = new BlinkingLed_FpgaInOutput();
    //end::ioPins[]
    
    /**This module builds the central ce signal.
     * Note: Modules which are organized in a strong tree hierarchy can be initialized by construction. 
     * This module is strongly treed, needs only the ioPins. 
     * Same is with next module {@link #res} which needs this module {@link CeTime_ifc}
     * but without circular aggregations.
     */
    public final ClockDivider ce = new ClockDivider(this.ioPins.reset_Inpin, this.ct);
    
    /**Build a reset signal high active for reset. Initial or also with the reset_Pin. 
     * This module is immediately connected to one of the inputFpga pins
     * via specific interface, see constructor argument type.
     */
    public final ResetCe res = new ResetCe(this.ioPins.reset_Inpin, this.ce.ce);
    
    public final Test_Combinatoric_BlinkingLed vhdl_Combinatoric = new Test_Combinatoric_BlinkingLed();
    
    public final BlinkingLedCt ct = new BlinkingLedCt(this.res, BlinkingLed_Fpga.this.blinkingLedCfg, this.ce);    //cfg implemented in extra class in this file.
    
    Modules ( ) {
      //aggregate the module afterwards
      this.ct.init(this.res, BlinkingLed_Fpga.this.blinkingLedCfg, this.ce);    //cfg implemented in extra class in this file.
    }
  }

  public final Modules modules;
  //end::Modules[]

  
  
  @Fpga.LINK_VHDL_MODULE static class Vhdlink_vhdl_Combinatoric {
    final boolean S, R;
    final boolean Q1, Q2;
    
    //Vhdlink_vhdl_Combinatoric ( ) { this.S = this.R = this.Q1 = this.Q2 = false; }
    
    @Fpga.LINK_VHDL_MODULE Vhdlink_vhdl_Combinatoric ( int time, BlinkingLed_Fpga thism, Test_Combinatoric_BlinkingLed vhdlMdl) {
      // firstly assign combinatoric values to internal variables
      this.S = Fpga.getBit(thism.modules.ct.q.ctLow, 13) && Fpga.getBit(thism.modules.ct.q.ctLow, 11);
      this.R = Fpga.getBit(thism.modules.ct.q.ctLow, 13) && Fpga.getBit(thism.modules.ct.q.ctLow, 10);
      vhdlMdl.input.S = this.S;             // VHDL can only associate variables to ports, not expressions
      vhdlMdl.input.R = this.R;             // This are the inputs of the PORT MAP
      vhdlMdl.step(time);
      vhdlMdl.update();
      // after calling step, get the outputs from the module to the internal variables for access.
      this.Q1 = vhdlMdl.output.Q1;     // VHDL cannot access the included module anywhere other than in the PORT MAP definition. 
      this.Q2 = vhdlMdl.output.Q2;     // This are the outputs of the PORT MAP
    }
    
  }
  
  /**Because it is combinatoric, a temporary _d class is not used. Update does nothing. 
   * 
   */
  Vhdlink_vhdl_Combinatoric vhdlink_vhdl_Combinatoric;
  
  
  /**This process should only build a 1 bit test output to select internal signals
   * to a test output pin of the FPGA.
   * The test pin is clocked, so that the path is not critical over all.
   * All input signals switches with CE0
   */
  @Fpga.VHDL_PROCESS public static class TestOut {

    public final boolean testOut1, testOut2, testOut3;
    
    TestOut(){
      this.testOut1 = false;
      this.testOut2 = false;
      this.testOut3 = false;
    }
    
    
    /**This process works with the full clock speed because of less functinality
     * and for a fast output signal. 
     * Possible may be also CE with CE1 and CE6
     * @param z
     * @param in
     * @param mdl
     */
    @Fpga.VHDL_PROCESS TestOut(TestOut z, Modules modules) {
      @Fpga.BITVECTOR(14) int sel; @Fpga.BITVECTOR(16) int chk;
      sel = 0b00000000000000;
      chk = 0x4100;              //should be come from any parameterizing. output selection.
      sel = Fpga.setBit(sel, 7, false); 
      sel = Fpga.setBit(sel, 8, Fpga.getBit(modules.ct.ct(), 0)); 
      this.testOut1 = (sel & Fpga.getBits(chk, 13, 0 ))!=0b00000000000000 && Fpga.getBits(chk, 15, 14 ) == 0b01;
      this.testOut2 = false; 
      this.testOut3 = false;
    }
  }

  TestOut testOut_d, testOut = new TestOut();

  
  //tag::ctor[]
  public BlinkingLed_Fpga ( ) {
    this.modules = new Modules();   //hint: should be invoke after ctor parts in class members, especially interface access agents should be set.
  }
  //end::ctor[]
  
  
  @Override public void reset ( ) {
    this.vhdlink_vhdl_Combinatoric = new Vhdlink_vhdl_Combinatoric(0, this, this.modules.vhdl_Combinatoric);
    this.modules.res.reset();
    this.modules.ce.reset();
    this.modules.ct.reset();
  }
  
  //tag::step_update[]  
  @Override
  public void step(int time) {
    this.vhdlink_vhdl_Combinatoric = new Vhdlink_vhdl_Combinatoric(time, this, this.modules.vhdl_Combinatoric);
    this.modules.res.step(time);
    this.modules.ce.step(time);
    this.modules.ct.step(time);
    
  }


  @Override
  public void update() {
    this.modules.res.update();
    this.modules.ce.update();
    this.modules.ct.update();
    //tag::outPins[]
    //end::outPins[]
  }
  //end::step_update[]  

  
  public void output ( ) {
    this.modules.ioPins.output.led1 = this.modules.ct.ledBlinking();
    this.modules.ioPins.output.led2 = this.modules.ct.getLedFast.getBit();
    this.modules.ioPins.output.led3 = Fpga.getBits(this.modules.ct.ct(), 2,0) != 0b000;
    this.modules.ioPins.output.led4 = this.modules.ct.getStateFast.getBit();
    this.modules.ioPins.output.led5 = this.modules.ct.vhdlink_ram.test3;
    this.modules.ioPins.output.led6 = this.vhdlink_vhdl_Combinatoric.Q1;
    this.modules.ioPins.output.led7 = this.vhdlink_vhdl_Combinatoric.Q2;
    this.modules.ioPins.output.led8 = this.testOut.testOut1; //this.modules.ct.ramUse.outRam;
  }
  
  
  //tag::time_BlinkingLed_ifcAccess[]

  /**Provides the used possibility for configuration values.
   */
  @Fpga.IfcAccess BlinkingLedCfg_ifc blinkingLedCfg = new BlinkingLedCfg_ifc ( ) {
    
    @Override @Fpga.BITVECTOR(8) public int time_BlinkingLed() {
      return 0x64;
    }

    @Override public int onDuration_BlinkingLed() {
      return 10;
    }

    @Override
    public int time() { return 0; }  // set from beginning

  };
  //end::time_BlinkingLed_ifcAccess[]


  
}


