package org.vishia.fpga.exmplBlinkingLed.modules;

import org.vishia.fpga.stdmodules.CeTime_ifc;
import org.vishia.fpga.stdmodules.Reset;
import org.vishia.fpga.stdmodules.Reset_Inpin_ifc;
import org.vishia.fpga.stdmodules.Reset_ifc;
import org.vishia.fpga.Fpga;
import org.vishia.fpga.FpgaModule_ifc;

public class ClockDivider implements FpgaModule_ifc {

  
  static class Ref {
    final Reset_Inpin_ifc resetInp;
    final BlinkingLed_ifc test;

    public Ref(Reset_Inpin_ifc resetInp, BlinkingLed_ifc test) {
      this.resetInp = resetInp; this.test = test;
    }
  }
  private Ref ref;
  
  //tag::Q-data[]
  /**Inner PROCESS class builds a TYPEDEF RECORD in VHDL and a PROCESS for each instance.
   * Note: Need public because here the interface technology is not used (negative, but simple understandable pattern).
   * Compare with {@link Reset.Q}
   */ 
  @Fpga.VHDL_PROCESS protected static final class Q{

    @Fpga.STDVECTOR(4) final int ct;
  
    /**This is the variable of the record accessed from outside. 
     * Note: Need public because here the interface technology is not used (negative, but simple understandable pattern).
     * Compare with {@link Reset.Q#res}
     */
    final boolean ce;  
  
    /**Time of the latest set operation of any of the variables. */
    int time_;
    //end::Q-data[]
    
    Q() {
      this.ct = 0;
      this.ce = false;
    }
    
    /**
     * @param time
     * @param z
     * @param ref
     */
    @Fpga.VHDL_PROCESS Q ( int time, Q z, Ref ref) {
      if(ref.resetInp.reset_Pin() == false) {
        this.ct = 0b0000;
        this.ce = false;
      }
      else if(z.ct < 0b1001) {                        // divides the system clock by 10
        this.ct = (z.ct +1);
        this.ce = false;
      } 
      else {
        this.ct = 0b0000;
        this.ce = true;
        this.time_ = time;
      }
    }

  }

  
  
  /**Instance of the current value of the PROCESS inner class.
   * Note: Should be public because here the interface technology is not used (negative, but simple understandable pattern).
   */
  protected Q q = new Q();
  private Q d_q;
  
  
  /**Module constructor with public access to instantiate.
   */
  public ClockDivider ( Reset_Inpin_ifc resetInp, BlinkingLed_ifc test ) {
    this.ref = new Ref(resetInp, test);
  }
  
  public ClockDivider ( ) {}
  
  public void init ( Reset_Inpin_ifc resetInp, BlinkingLed_ifc test ) {
    this.ref = new Ref(resetInp, test);
  }
  
  
  @Override
  public void reset() {
    this.q = new Q();
  }

  @Override
  public void step(int time) {
    this.d_q = new Q(time, this.q, this.ref);
  }

  @Override
  public void update() {
    this.q = this.d_q;
  }

  @Fpga.IfcAccess public CeTime_ifc ce = new CeTime_ifc() {
    @Override public boolean ce () { return ClockDivider.this.q.ce; }

    @Override public int time () {
      return ClockDivider.this.q.time_;
    }

    @Override public int period () {
     return 10;
    }

    @Override public String timeGroupName () {
      return "CE";
    }
    
  };
  
  
}
