library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;

ENTITY Test_MyFpga IS
PORT (
  clk: IN BIT;
  reset_Pin : IN BIT;
  tristatePin : INOUT STD_LOGIC;
  data : OUT BIT_VECTOR(15 DOWNTO 0);
  testOutFromModule : OUT BIT;
  testout : OUT BIT  --last line of port definition
);
END Test_MyFpga;

ARCHITECTURE BEHAVIORAL OF Test_MyFpga IS

TYPE ModuleXY_In_REC IS RECORD
  in1 : BIT;
  inVal : BIT_VECTOR(7 DOWNTO 0);
END RECORD ModuleXY_In_REC;

TYPE ModuleXY_Out_REC IS RECORD
  out1 : BIT;
  out2 : BIT;
  out3 : BIT;
  out4 : BIT;
  out5 : BIT;
  out6 : BIT;
  out7 : BIT;
  olg1 : STD_LOGIC;
  olg5 : STD_LOGIC;
  bVal1 : BIT_VECTOR(15 DOWNTO 0);
  bVal2 : BIT_VECTOR(15 DOWNTO 0);
  bVal3 : BIT_VECTOR(15 DOWNTO 0);
  oVal1 : STD_LOGIC_VECTOR(15 DOWNTO 0);
  oVal2 : STD_LOGIC_VECTOR(15 DOWNTO 0);
  oVal3 : STD_LOGIC_VECTOR(15 DOWNTO 0);
  oVal4 : STD_LOGIC_VECTOR(15 DOWNTO 0);
  oVal5 : STD_LOGIC_VECTOR(15 DOWNTO 0);
  oVal6 : STD_LOGIC_VECTOR(15 DOWNTO 0);
  oVal7 : STD_LOGIC_VECTOR(15 DOWNTO 0);
  oVal8 : STD_LOGIC_VECTOR(15 DOWNTO 0);
  oVal9 : STD_LOGIC_VECTOR(15 DOWNTO 0);
  oPins : STD_LOGIC_VECTOR(15 DOWNTO 0);
  oVec81 : BIT_VECTOR(7 DOWNTO 0);
  oVec82 : BIT_VECTOR(7 DOWNTO 0);
  oVec83 : BIT_VECTOR(7 DOWNTO 0);
  oVec84 : BIT_VECTOR(7 DOWNTO 0);
  oVec85 : BIT_VECTOR(7 DOWNTO 0);
  oVec86 : BIT_VECTOR(7 DOWNTO 0);
  oVec87 : BIT_VECTOR(7 DOWNTO 0);
  oVec88 : BIT_VECTOR(7 DOWNTO 0);
  oVec89 : BIT_VECTOR(7 DOWNTO 0);
END RECORD ModuleXY_Out_REC;

TYPE ModuleXY_PCE_REC IS RECORD
  ce : BIT;
  ce7 : BIT;
  ct : STD_LOGIC_VECTOR(3 DOWNTO 0);
END RECORD ModuleXY_PCE_REC;

TYPE ModuleXY_Q_CE0_REC IS RECORD
  var1 : BIT;
  val : STD_LOGIC_VECTOR(15 DOWNTO 0);
END RECORD ModuleXY_Q_CE0_REC;

TYPE ModuleXY_Val_CE7_REC IS RECORD
  bit1 : BIT;
  bit2 : BIT;
  bit3 : BIT;
  lg1 : STD_LOGIC;
  lg2 : STD_LOGIC;
  lg3 : STD_LOGIC;
  lc1 : STD_LOGIC;
  lc2 : STD_LOGIC;
  lc3 : STD_LOGIC;
  val1 : STD_LOGIC_VECTOR(15 DOWNTO 0);
  val2 : STD_LOGIC_VECTOR(15 DOWNTO 0);
  val3 : STD_LOGIC_VECTOR(15 DOWNTO 0);
  bvec1 : BIT_VECTOR(15 DOWNTO 0);
  bvec2 : BIT_VECTOR(15 DOWNTO 0);
END RECORD ModuleXY_Val_CE7_REC;

TYPE OtherModuleXz_Qx_REC IS RECORD
  bit1 : BIT;
END RECORD OtherModuleXz_Qx_REC;

TYPE Reset_Q_REC IS RECORD
  resetCount : STD_LOGIC_VECTOR(3 DOWNTO 0);
  res : BIT;
END RECORD Reset_Q_REC;

TYPE SubmoduleXz_In_REC IS RECORD
  var : BIT;
END RECORD SubmoduleXz_In_REC;

TYPE SubmoduleXz_Out_REC IS RECORD
  outX1 : BIT;
  outVal : STD_LOGIC_VECTOR(7 DOWNTO 0);
END RECORD SubmoduleXz_Out_REC;

-- == Definition of record instances for the processes:
-- VHDL-Syntax:          SIGNAL <signalName> : <typeName>
--SIGNAL {<&nameModule>_}<&nameProcessClass> : <&nameClass>_<&nameProcessClass>_REC
--   ... <&nameModule> is defined in  class Modules{...}  in top level Java file, name of the module instance.
--   ... <&nameModule> for Process classes in toplevel: name of the top level class and Java file (start with upper case).
--   ... more <&nameModule> can be defined in  class Modules{...}  in the module Java file, nested modules.
--   ... <&nameProcessClass> is the name of the inner class in the Java module, same as instance name of the process
--   ... <&nameClass> is the type name of the modul's class
SIGNAL mdl1_In : ModuleXY_In_REC;
SIGNAL mdl1_Out : ModuleXY_Out_REC;
SIGNAL mdl1_PCE : ModuleXY_PCE_REC;
SIGNAL mdl1_Q_CE0 : ModuleXY_Q_CE0_REC;
SIGNAL mdl1_Val_CE7 : ModuleXY_Val_CE7_REC;
SIGNAL mdl1_submdlXz_In : SubmoduleXz_In_REC;
SIGNAL mdl1_submdlXz_Out : SubmoduleXz_Out_REC;
SIGNAL module2_Qx : OtherModuleXz_Qx_REC;
SIGNAL reset_Q : Reset_Q_REC;

CONSTANT ModuleXY_srcCE0_period : INTEGER := 10;
CONSTANT ModuleXY_srcCE7_period : INTEGER := 10;
CONSTANT OtherModuleXz_getConstValue : BIT_VECTOR(15 DOWNTO 0) := x"3456";
CONSTANT Reset_stubFalse_getBit : BIT := '0';
CONSTANT Reset_stubTrue_getBit : BIT := '1';
CONSTANT Reset_stubWord16_getWord : BIT_VECTOR(15 DOWNTO 0) := x"0000";



BEGIN

--Assignments from all input() operations to set ..In data of modules

mdl1_In.in1  <=  '1' WHEN  (tristatePin = '1' )  ELSE '0';
mdl1_submdlXz_In.var <=  module2_Qx.bit1;


ioPins_Output_data_PRC: PROCESS ( clk )
BEGIN IF(clk'event AND clK='1') THEN

  IF (mdl1_PCE.ce)='1' THEN
      data <= TO_BITVECTOR(mdl1_Out.oPins);
  ELSE
  END IF;
END IF; END PROCESS;



mdl1_PCE_PRC: PROCESS ( clk )
BEGIN IF(clk'event AND clK='1') THEN

  IF mdl1_PCE.ct(3 DOWNTO 2) =  "11"  THEN
      mdl1_PCE.ct <=   "1000";
      mdl1_PCE.ce <=  '1';
  ELSE
      mdl1_PCE.ct <=  mdl1_PCE.ct - 1 ;
      mdl1_PCE.ce <=  '0';
  END IF;
  IF mdl1_PCE.ct =  "0010"  THEN
      mdl1_PCE.ce7 <=  '1';
  ELSE
      mdl1_PCE.ce7 <=  '0';
  END IF;
END IF; END PROCESS;



mdl1_Q_CE0_PRC: PROCESS ( clk )
BEGIN IF(clk'event AND clK='1') THEN

  IF (mdl1_PCE.ce)='1' THEN
      IF (reset_Q.res)='1' THEN
          mdl1_Q_CE0.var1 <=  '0';
          mdl1_Q_CE0.val <=  x"1234";
      ELSE
          mdl1_Q_CE0.val <=  mdl1_Q_CE0.val + 1 ;
          IF mdl1_Q_CE0.val = x"0010"  THEN mdl1_Q_CE0.var1  <=  '1'; ELSE mdl1_Q_CE0.var1  <=  '0'; END IF;
      END IF;
  ELSE
  END IF;
END IF; END PROCESS;



mdl1_Val_CE7_PRC: PROCESS ( clk )
  VARIABLE localVar1 : BIT;
BEGIN IF(clk'event AND clK='1') THEN

  localVar1 := reset_Pin;
  IF (mdl1_PCE.ce7)='1' THEN
      IF (reset_Q.res)='1' THEN
          localVar1 :=  '0';
          mdl1_Val_CE7.bit1 <=  '0';
          mdl1_Val_CE7.bit2 <=  '0';
          mdl1_Val_CE7.bit3 <=  '0';
          mdl1_Val_CE7.lg1 <=  '0';
          mdl1_Val_CE7.lg2 <=  '0';
          mdl1_Val_CE7.lg3 <=  '0';
          mdl1_Val_CE7.lc1 <=  '0';
          mdl1_Val_CE7.lc2 <=  '0';
          mdl1_Val_CE7.lc3 <=  '0';
          mdl1_Val_CE7.val1 <=  x"0000";
          mdl1_Val_CE7.val2 <=  x"0000";
          mdl1_Val_CE7.val3 <=  x"0000";
          mdl1_Val_CE7.bvec1 <=  x"0000";
          mdl1_Val_CE7.bvec2 <=  x"0000";
      ELSE
          mdl1_Val_CE7.bit1 <=  mdl1_Val_CE7.bit1 AND localVar1  AND module2_Qx.bit1 ;
          mdl1_Val_CE7.bit2 <=  mdl1_PCE.ce;
          mdl1_Val_CE7.val1 <= TO_STDLOGICVECTOR(OtherModuleXz_getConstValue);
          mdl1_Val_CE7.lg1 <= TO_STDULOGIC(mdl1_Val_CE7.bit1 AND TO_BIT(mdl1_Val_CE7.lg1) );
          mdl1_Val_CE7.lg2 <=  mdl1_Val_CE7.lg2 AND mdl1_Val_CE7.lg3 ;
          mdl1_Val_CE7.lg3 <=  mdl1_Val_CE7.lg2 AND mdl1_Val_CE7.lg3 ;
          IF (mdl1_Val_CE7.bit1 AND TO_BIT(mdl1_Val_CE7.lg1) )='1' THEN mdl1_Val_CE7.lc1  <= '1';
          ELSE mdl1_Val_CE7.lc1  <= '0'; END IF;
          IF (mdl1_Val_CE7.bit1 AND TO_BIT(mdl1_Val_CE7.lg1) )='1' THEN mdl1_Val_CE7.lc2  <= '1';
          ELSE mdl1_Val_CE7.lc2  <= '0'; END IF;
          IF (mdl1_Val_CE7.bit1 AND TO_BIT(mdl1_Val_CE7.lg1) )='1' THEN mdl1_Val_CE7.lc3  <= '1';
          ELSE mdl1_Val_CE7.lc3  <= '0'; END IF;
          IF module2_Qx.bit1='1' THEN mdl1_Val_CE7.val2  <= x"fedc";
          ELSE mdl1_Val_CE7.val2  <= mdl1_Q_CE0.val; END IF;
          mdl1_Val_CE7.val3 <=  NOT mdl1_Val_CE7.val2;
          mdl1_Val_CE7.bvec1 <=  mdl1_Val_CE7.bvec1 OR mdl1_Val_CE7.bvec2 ;
          IF module2_Qx.bit1='1' THEN mdl1_Val_CE7.bvec2  <= x"1248";
          ELSE mdl1_Val_CE7.bvec2  <= x"248c"; END IF;
      END IF;
  ELSE
  END IF;
END IF; END PROCESS;



module2_Qx_PRC: PROCESS ( clk )
BEGIN IF(clk'event AND clK='1') THEN

  module2_Qx.bit1 <=  NOT module2_Qx.bit1;
END IF; END PROCESS;



reset_Q_PRC: PROCESS ( clk )
BEGIN IF(clk'event AND clK='1') THEN

  IF reset_Pin = '0'  THEN
      reset_Q.resetCount <=   "0000";
  ELSE
    IF reset_Q.res='1' THEN
        reset_Q.resetCount <=  reset_Q.resetCount + 1 ;
    ELSE
    END IF;
  END IF;
  IF reset_Q.resetCount <  "1110"  THEN reset_Q.res  <=  '1'; ELSE reset_Q.res  <=  '0'; END IF;
END IF; END PROCESS;

--Assignments from all output() operations to set Out data of modules and set FPGA out pins

testout <=  mdl1_Q_CE0.var1;
tristatePin <=  mdl1_Out.olg1;
mdl1_Out.out1 <=  mdl1_Q_CE0.var1 AND mdl1_submdlXz_Out.outX1 ;
testOutFromModule <=  '1';
mdl1_Out.out2 <=  mdl1_Val_CE7.bit3 AND mdl1_Val_CE7.bit2 ;
mdl1_Out.out3 <= TO_BIT(mdl1_Val_CE7.lg1 AND TO_STDULOGIC(mdl1_Val_CE7.bit2) );

mdl1_Out.olg1  <= '1'  WHEN (mdl1_Val_CE7.lg1 AND TO_STDULOGIC(mdl1_Val_CE7.bit2) )='1' ELSE '0' ;
mdl1_Out.out4 <=   (mdl1_Val_CE7.bit1 AND mdl1_Val_CE7.bit2 )  OR mdl1_Val_CE7.bit3 ;
mdl1_Out.out5  <=  '1' WHEN mdl1_Val_CE7.bvec1 = x"4567"  ELSE '0';
mdl1_Out.out6  <=  '1' WHEN mdl1_Val_CE7.val1 < mdl1_Val_CE7.val2  AND  ( ( (mdl1_Val_CE7.bvec1 AND TO_BITVECTOR(mdl1_Val_CE7.val1) ) )  > x"0005" )   ELSE '0';
mdl1_Out.oVal1 <=  mdl1_Val_CE7.val1 + mdl1_Val_CE7.val2 ;
mdl1_Out.oVal2 <=  mdl1_Val_CE7.val1 + TO_STDLOGICVECTOR(mdl1_Val_CE7.bvec2) ;
mdl1_Out.bVal2 <=  mdl1_Val_CE7.bvec2 SRL 2 ;
mdl1_Out.bVal3 <=  mdl1_Val_CE7.bvec2 SLL 3  OR x"0003" ;
mdl1_Out.out7 <= TO_BIT(mdl1_Val_CE7.val1(5));
mdl1_Out.oVec81 <= TO_BITVECTOR(mdl1_Val_CE7.val1(13 DOWNTO 6));
mdl1_Out.oVec82 <=  mdl1_Out.oVec81(2 DOWNTO 0) & mdl1_Out.oVec82(5 DOWNTO 1);
mdl1_Out.oVal4 <=  mdl1_Val_CE7.val1(14 DOWNTO 0) & ('0');
mdl1_Out.oVal5 <=  ('1') & mdl1_Val_CE7.val1(15 DOWNTO 1);
mdl1_Out.oVal6 <= TO_STDLOGICVECTOR(mdl1_Out.oVec81 & mdl1_Out.oVec81);
mdl1_Out.oVal3 <= TO_STDLOGICVECTOR(mdl1_Out.oVec81 & "00" & mdl1_Out.oVec82(6 DOWNTO 1));
mdl1_Out.oVal8 <= TO_STDLOGICVECTOR( "000" & mdl1_Out.oVec81(6 DOWNTO 0) & mdl1_Out.oVec82(7 DOWNTO 2));
mdl1_Out.oPins <=  mdl1_Out.oVal1 OR mdl1_Out.oVal2  OR mdl1_Out.oVal3  OR mdl1_Out.oVal4  OR mdl1_Out.oVal5  OR mdl1_Out.oVal6 ;
mdl1_submdlXz_Out.outX1 <=  '0';
mdl1_submdlXz_Out.outVal <=  x"c3";

END BEHAVIORAL;
